"use strict";
var path = require('path');
var compiler = require('@angular/compiler');
var core_1 = require('@angular/core');
var static_reflector_1 = require('./static_reflector');
var metadata_resolver_1 = require('@angular/compiler/src/metadata_resolver');
var html_parser_1 = require('@angular/compiler/src/html_parser');
var directive_normalizer_1 = require('@angular/compiler/src/directive_normalizer');
var lexer_1 = require('@angular/compiler/src/expression_parser/lexer');
var parser_1 = require('@angular/compiler/src/expression_parser/parser');
var template_parser_1 = require('@angular/compiler/src/template_parser');
var dom_element_schema_registry_1 = require('@angular/compiler/src/schema/dom_element_schema_registry');
var style_compiler_1 = require('@angular/compiler/src/style_compiler');
var view_compiler_1 = require('@angular/compiler/src/view_compiler/view_compiler');
var ts_emitter_1 = require('@angular/compiler/src/output/ts_emitter');
var platform_server_1 = require('@angular/platform-server');
var reflector_host_1 = require('./reflector_host');
var GENERATED_FILES = /\.ngfactory\.ts$|\.css\.ts$|\.css\.shim\.ts$/;
var PREAMBLE = "/**\n * This file is generated by the Angular 2 template compiler.\n * Do not edit.\n */\n";
var CodeGenerator = (function () {
    function CodeGenerator(ngOptions, basePath, program, host, staticReflector, resolver, compiler, reflectorHost) {
        this.ngOptions = ngOptions;
        this.basePath = basePath;
        this.program = program;
        this.host = host;
        this.staticReflector = staticReflector;
        this.resolver = resolver;
        this.compiler = compiler;
        this.reflectorHost = reflectorHost;
    }
    CodeGenerator.prototype.generateSource = function (metadatas) {
        var _this = this;
        var normalize = function (metadata) {
            var directiveType = metadata.type.runtime;
            var directives = _this.resolver.getViewDirectivesMetadata(directiveType);
            var pipes = _this.resolver.getViewPipesMetadata(directiveType);
            return new compiler.NormalizedComponentWithViewDirectives(metadata, directives, pipes);
        };
        return this.compiler.compileTemplates(metadatas.map(normalize));
    };
    CodeGenerator.prototype.readComponents = function (absSourcePath) {
        var result = [];
        var metadata = this.staticReflector.getModuleMetadata(absSourcePath);
        if (!metadata) {
            console.log("WARNING: no metadata found for " + absSourcePath);
            return result;
        }
        var symbols = Object.keys(metadata['metadata']);
        if (!symbols || !symbols.length) {
            return result;
        }
        for (var _i = 0, symbols_1 = symbols; _i < symbols_1.length; _i++) {
            var symbol = symbols_1[_i];
            var staticType = this.reflectorHost.findDeclaration(absSourcePath, symbol, absSourcePath);
            var directive = void 0;
            directive = this.resolver.maybeGetDirectiveMetadata(staticType);
            if (!directive || !directive.isComponent) {
                continue;
            }
            result.push(this.compiler.normalizeDirectiveMetadata(directive));
        }
        return result;
    };
    // TODO(tbosch): add a cache for shared css files
    // TODO(tbosch): detect cycles!
    CodeGenerator.prototype.generateStylesheet = function (filepath, shim) {
        var _this = this;
        return this.compiler.loadAndCompileStylesheet(filepath, shim, '.ts')
            .then(function (sourceWithImports) {
            // Write codegen in a directory structure matching the sources.
            // TODO(alexeagle): relativize paths by the rootDirs option
            var emitPath = path.join(_this.ngOptions.genDir, path.relative(_this.basePath, sourceWithImports.source.moduleUrl));
            _this.host.writeFile(emitPath, PREAMBLE + sourceWithImports.source.source, false);
            return Promise.all(sourceWithImports.importedUrls.map(function (url) { return _this.generateStylesheet(url, shim); }));
        });
    };
    CodeGenerator.prototype.codegen = function () {
        var _this = this;
        platform_server_1.Parse5DomAdapter.makeCurrent();
        var generateOneFile = function (absSourcePath) {
            return Promise.all(_this.readComponents(absSourcePath))
                .then(function (metadatas) {
                if (!metadatas || !metadatas.length) {
                    return;
                }
                var stylesheetPromises = [];
                metadatas.forEach(function (metadata) {
                    var stylesheetPaths = metadata && metadata.template && metadata.template.styleUrls;
                    if (stylesheetPaths) {
                        stylesheetPaths.forEach(function (path) {
                            stylesheetPromises.push(_this.generateStylesheet(path, metadata.template.encapsulation === core_1.ViewEncapsulation.Emulated));
                        });
                    }
                });
                var generated = _this.generateSource(metadatas);
                var sourceFile = _this.program.getSourceFile(absSourcePath);
                // Write codegen in a directory structure matching the sources.
                // TODO(alexeagle): relativize paths by the rootDirs option
                var emitPath = path.join(_this.ngOptions.genDir, path.relative(_this.basePath, generated.moduleUrl));
                _this.host.writeFile(emitPath, PREAMBLE + generated.source, false, function () { }, [sourceFile]);
                return Promise.all(stylesheetPromises);
            })
                .catch(function (e) { console.error(e.stack); });
        };
        return Promise.all(this.program.getRootFileNames().filter(function (f) { return !GENERATED_FILES.test(f); }).map(generateOneFile));
    };
    CodeGenerator.create = function (ngOptions, program, options, compilerHost) {
        var xhr = { get: function (s) { return Promise.resolve(compilerHost.readFile(s)); } };
        var urlResolver = compiler.createOfflineCompileUrlResolver();
        var reflectorHost = new reflector_host_1.NodeReflectorHost(program, compilerHost, options, ngOptions);
        var staticReflector = new static_reflector_1.StaticReflector(reflectorHost);
        var htmlParser = new html_parser_1.HtmlParser();
        var normalizer = new directive_normalizer_1.DirectiveNormalizer(xhr, urlResolver, htmlParser);
        var parser = new parser_1.Parser(new lexer_1.Lexer());
        var tmplParser = new template_parser_1.TemplateParser(parser, new dom_element_schema_registry_1.DomElementSchemaRegistry(), htmlParser, 
        /*console*/ null, []);
        var offlineCompiler = new compiler.OfflineCompiler(normalizer, tmplParser, new style_compiler_1.StyleCompiler(urlResolver), new view_compiler_1.ViewCompiler(new compiler.CompilerConfig(true, true, true)), new ts_emitter_1.TypeScriptEmitter(reflectorHost), xhr);
        var resolver = new metadata_resolver_1.CompileMetadataResolver(new compiler.DirectiveResolver(staticReflector), new compiler.PipeResolver(staticReflector), new compiler.ViewResolver(staticReflector), null, null, staticReflector);
        return new CodeGenerator(ngOptions, ngOptions.basePath, program, compilerHost, staticReflector, resolver, offlineCompiler, reflectorHost);
    };
    return CodeGenerator;
}());
exports.CodeGenerator = CodeGenerator;
//# sourceMappingURL=codegen.js.map