/// <reference path="../../typings/jasmine/jasmine.d.ts" />
"use strict";
var testing_internal_1 = require('@angular/core/testing/testing_internal');
var lang_1 = require('@angular/facade/src/lang');
var collection_1 = require('@angular/facade/src/collection');
var static_reflector_1 = require('./static_reflector');
testing_internal_1.describe('StaticReflector', function () {
    var noContext = new static_reflector_1.StaticSymbol('', '');
    var host;
    var reflector;
    testing_internal_1.beforeEach(function () {
        host = new MockReflectorHost();
        reflector = new static_reflector_1.StaticReflector(host);
    });
    function simplify(context, value) {
        return reflector.simplify(context, value);
    }
    testing_internal_1.it('should get annotations for NgFor', function () {
        var NgFor = host.findDeclaration('angular2/src/common/directives/ng_for', 'NgFor');
        var annotations = reflector.annotations(NgFor);
        testing_internal_1.expect(annotations.length).toEqual(1);
        var annotation = annotations[0];
        testing_internal_1.expect(annotation.selector).toEqual('[ngFor][ngForOf]');
        testing_internal_1.expect(annotation.inputs).toEqual(['ngForTrackBy', 'ngForOf', 'ngForTemplate']);
    });
    testing_internal_1.it('should get constructor for NgFor', function () {
        var NgFor = host.findDeclaration('angular2/src/common/directives/ng_for', 'NgFor');
        var ViewContainerRef = host.findDeclaration('angular2/src/core/linker/view_container_ref', 'ViewContainerRef');
        var TemplateRef = host.findDeclaration('angular2/src/core/linker/template_ref', 'TemplateRef');
        var IterableDiffers = host.findDeclaration('angular2/src/core/change_detection/differs/iterable_differs', 'IterableDiffers');
        var ChangeDetectorRef = host.findDeclaration('angular2/src/core/change_detection/change_detector_ref', 'ChangeDetectorRef');
        var parameters = reflector.parameters(NgFor);
        testing_internal_1.expect(parameters)
            .toEqual([[ViewContainerRef], [TemplateRef], [IterableDiffers], [ChangeDetectorRef]]);
    });
    testing_internal_1.it('should get annotations for HeroDetailComponent', function () {
        var HeroDetailComponent = host.findDeclaration('src/app/hero-detail.component', 'HeroDetailComponent');
        var annotations = reflector.annotations(HeroDetailComponent);
        testing_internal_1.expect(annotations.length).toEqual(1);
        var annotation = annotations[0];
        testing_internal_1.expect(annotation.selector).toEqual('my-hero-detail');
        testing_internal_1.expect(annotation.directives)
            .toEqual([[host.findDeclaration('angular2/src/common/directives/ng_for', 'NgFor')]]);
    });
    testing_internal_1.it('should get and empty annotation list for an unknown class', function () {
        var UnknownClass = host.findDeclaration('src/app/app.component', 'UnknownClass');
        var annotations = reflector.annotations(UnknownClass);
        testing_internal_1.expect(annotations).toEqual([]);
    });
    testing_internal_1.it('should get propMetadata for HeroDetailComponent', function () {
        var HeroDetailComponent = host.findDeclaration('src/app/hero-detail.component', 'HeroDetailComponent');
        var props = reflector.propMetadata(HeroDetailComponent);
        testing_internal_1.expect(props['hero']).toBeTruthy();
    });
    testing_internal_1.it('should get an empty object from propMetadata for an unknown class', function () {
        var UnknownClass = host.findDeclaration('src/app/app.component', 'UnknownClass');
        var properties = reflector.propMetadata(UnknownClass);
        testing_internal_1.expect(properties).toEqual({});
    });
    testing_internal_1.it('should get empty parameters list for an unknown class ', function () {
        var UnknownClass = host.findDeclaration('src/app/app.component', 'UnknownClass');
        var parameters = reflector.parameters(UnknownClass);
        testing_internal_1.expect(parameters).toEqual([]);
    });
    testing_internal_1.it('should simplify primitive into itself', function () {
        testing_internal_1.expect(simplify(noContext, 1)).toBe(1);
        testing_internal_1.expect(simplify(noContext, true)).toBe(true);
        testing_internal_1.expect(simplify(noContext, "some value")).toBe("some value");
    });
    testing_internal_1.it('should simplify an array into a copy of the array', function () { testing_internal_1.expect(simplify(noContext, [1, 2, 3])).toEqual([1, 2, 3]); });
    testing_internal_1.it('should simplify an object to a copy of the object', function () {
        var expr = { a: 1, b: 2, c: 3 };
        testing_internal_1.expect(simplify(noContext, expr)).toEqual(expr);
    });
    testing_internal_1.it('should simplify &&', function () {
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '&&', left: true, right: true }))).toBe(true);
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '&&', left: true, right: false }))).toBe(false);
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '&&', left: false, right: true }))).toBe(false);
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '&&', left: false, right: false }))).toBe(false);
    });
    testing_internal_1.it('should simplify ||', function () {
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '||', left: true, right: true }))).toBe(true);
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '||', left: true, right: false }))).toBe(true);
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '||', left: false, right: true }))).toBe(true);
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '||', left: false, right: false }))).toBe(false);
    });
    testing_internal_1.it('should simplify &', function () {
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '&', left: 0x22, right: 0x0F }))).toBe(0x22 & 0x0F);
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '&', left: 0x22, right: 0xF0 }))).toBe(0x22 & 0xF0);
    });
    testing_internal_1.it('should simplify |', function () {
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '|', left: 0x22, right: 0x0F }))).toBe(0x22 | 0x0F);
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '|', left: 0x22, right: 0xF0 }))).toBe(0x22 | 0xF0);
    });
    testing_internal_1.it('should simplify ^', function () {
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '|', left: 0x22, right: 0x0F }))).toBe(0x22 | 0x0F);
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '|', left: 0x22, right: 0xF0 }))).toBe(0x22 | 0xF0);
    });
    testing_internal_1.it('should simplify ==', function () {
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '==', left: 0x22, right: 0x22 }))).toBe(0x22 == 0x22);
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '==', left: 0x22, right: 0xF0 }))).toBe(0x22 == 0xF0);
    });
    testing_internal_1.it('should simplify !=', function () {
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '!=', left: 0x22, right: 0x22 }))).toBe(0x22 != 0x22);
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '!=', left: 0x22, right: 0xF0 }))).toBe(0x22 != 0xF0);
    });
    testing_internal_1.it('should simplify ===', function () {
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '===', left: 0x22, right: 0x22 }))).toBe(0x22 === 0x22);
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '===', left: 0x22, right: 0xF0 }))).toBe(0x22 === 0xF0);
    });
    testing_internal_1.it('should simplify !==', function () {
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '!==', left: 0x22, right: 0x22 }))).toBe(0x22 !== 0x22);
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '!==', left: 0x22, right: 0xF0 }))).toBe(0x22 !== 0xF0);
    });
    testing_internal_1.it('should simplify >', function () {
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '>', left: 1, right: 1 }))).toBe(1 > 1);
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '>', left: 1, right: 0 }))).toBe(1 > 0);
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '>', left: 0, right: 1 }))).toBe(0 > 1);
    });
    testing_internal_1.it('should simplify >=', function () {
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '>=', left: 1, right: 1 }))).toBe(1 >= 1);
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '>=', left: 1, right: 0 }))).toBe(1 >= 0);
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '>=', left: 0, right: 1 }))).toBe(0 >= 1);
    });
    testing_internal_1.it('should simplify <=', function () {
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '<=', left: 1, right: 1 }))).toBe(1 <= 1);
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '<=', left: 1, right: 0 }))).toBe(1 <= 0);
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '<=', left: 0, right: 1 }))).toBe(0 <= 1);
    });
    testing_internal_1.it('should simplify <', function () {
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '<', left: 1, right: 1 }))).toBe(1 < 1);
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '<', left: 1, right: 0 }))).toBe(1 < 0);
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '<', left: 0, right: 1 }))).toBe(0 < 1);
    });
    testing_internal_1.it('should simplify <<', function () {
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '<<', left: 0x55, right: 2 }))).toBe(0x55 << 2);
    });
    testing_internal_1.it('should simplify >>', function () {
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '>>', left: 0x55, right: 2 }))).toBe(0x55 >> 2);
    });
    testing_internal_1.it('should simplify +', function () {
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '+', left: 0x55, right: 2 }))).toBe(0x55 + 2);
    });
    testing_internal_1.it('should simplify -', function () {
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '-', left: 0x55, right: 2 }))).toBe(0x55 - 2);
    });
    testing_internal_1.it('should simplify *', function () {
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '*', left: 0x55, right: 2 }))).toBe(0x55 * 2);
    });
    testing_internal_1.it('should simplify /', function () {
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '/', left: 0x55, right: 2 }))).toBe(0x55 / 2);
    });
    testing_internal_1.it('should simplify %', function () {
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'binop', operator: '%', left: 0x55, right: 2 }))).toBe(0x55 % 2);
    });
    testing_internal_1.it('should simplify prefix -', function () {
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'pre', operator: '-', operand: 2 }))).toBe(-2);
    });
    testing_internal_1.it('should simplify prefix ~', function () {
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'pre', operator: '~', operand: 2 }))).toBe(~2);
    });
    testing_internal_1.it('should simplify prefix !', function () {
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'pre', operator: '!', operand: true }))).toBe(!true);
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: 'pre', operator: '!', operand: false }))).toBe(!false);
    });
    testing_internal_1.it('should simplify an array index', function () {
        testing_internal_1.expect(simplify(noContext, ({ __symbolic: "index", expression: [1, 2, 3], index: 2 }))).toBe(3);
    });
    testing_internal_1.it('should simplify an object index', function () {
        var expr = { __symbolic: "select", expression: { a: 1, b: 2, c: 3 }, member: "b" };
        testing_internal_1.expect(simplify(noContext, expr)).toBe(2);
    });
    testing_internal_1.it('should simplify a module reference', function () {
        testing_internal_1.expect(simplify(new static_reflector_1.StaticSymbol('/src/cases', ''), ({ __symbolic: "reference", module: "./extern", name: "s" })))
            .toEqual("s");
    });
    testing_internal_1.it('should simplify a non existing reference as a static symbol', function () {
        testing_internal_1.expect(simplify(new static_reflector_1.StaticSymbol('/src/cases', ''), ({ __symbolic: "reference", module: "./extern", name: "nonExisting" })))
            .toEqual(host.getStaticSymbol('/src/extern.d.ts', 'nonExisting'));
    });
});
var MockReflectorHost = (function () {
    function MockReflectorHost() {
        this.staticTypeCache = new Map();
    }
    MockReflectorHost.prototype.angularImportLocations = function () {
        return {
            coreDecorators: 'angular2/src/core/metadata',
            diDecorators: 'angular2/src/core/di/decorators',
            diMetadata: 'angular2/src/core/di/metadata',
            provider: 'angular2/src/core/di/provider'
        };
    };
    MockReflectorHost.prototype.getStaticSymbol = function (declarationFile, name) {
        var cacheKey = declarationFile + ":" + name;
        var result = this.staticTypeCache.get(cacheKey);
        if (lang_1.isBlank(result)) {
            result = new static_reflector_1.StaticSymbol(declarationFile, name);
            this.staticTypeCache.set(cacheKey, result);
        }
        return result;
    };
    // In tests, assume that symbols are not re-exported
    MockReflectorHost.prototype.findDeclaration = function (modulePath, symbolName, containingFile) {
        function splitPath(path) { return path.split(/\/|\\/g); }
        function resolvePath(pathParts) {
            var result = [];
            collection_1.ListWrapper.forEachWithIndex(pathParts, function (part, index) {
                switch (part) {
                    case '':
                    case '.':
                        if (index > 0)
                            return;
                        break;
                    case '..':
                        if (index > 0 && result.length != 0)
                            result.pop();
                        return;
                }
                result.push(part);
            });
            return result.join('/');
        }
        function pathTo(from, to) {
            var result = to;
            if (to.startsWith('.')) {
                var fromParts = splitPath(from);
                fromParts.pop(); // remove the file name.
                var toParts = splitPath(to);
                result = resolvePath(fromParts.concat(toParts));
            }
            return result;
        }
        if (modulePath.indexOf('.') === 0) {
            return this.getStaticSymbol(pathTo(containingFile, modulePath) + '.d.ts', symbolName);
        }
        return this.getStaticSymbol('/tmp/' + modulePath + '.d.ts', symbolName);
    };
    MockReflectorHost.prototype.getMetadataFor = function (moduleId) {
        var data = {
            '/tmp/angular2/src/common/forms/directives.d.ts': {
                "__symbolic": "module",
                "metadata": {
                    "FORM_DIRECTIVES": [
                        {
                            "__symbolic": "reference",
                            "name": "NgFor",
                            "module": "angular2/src/common/directives/ng_for"
                        }
                    ]
                }
            },
            '/tmp/angular2/src/common/directives/ng_for.d.ts': {
                "__symbolic": "module",
                "metadata": {
                    "NgFor": {
                        "__symbolic": "class",
                        "decorators": [
                            {
                                "__symbolic": "call",
                                "expression": {
                                    "__symbolic": "reference",
                                    "name": "Directive",
                                    "module": "../../core/metadata"
                                },
                                "arguments": [
                                    {
                                        "selector": "[ngFor][ngForOf]",
                                        "inputs": ["ngForTrackBy", "ngForOf", "ngForTemplate"]
                                    }
                                ]
                            }
                        ],
                        "members": {
                            "__ctor__": [
                                {
                                    "__symbolic": "constructor",
                                    "parameters": [
                                        {
                                            "__symbolic": "reference",
                                            "module": "../../core/linker/view_container_ref",
                                            "name": "ViewContainerRef"
                                        },
                                        {
                                            "__symbolic": "reference",
                                            "module": "../../core/linker/template_ref",
                                            "name": "TemplateRef"
                                        },
                                        {
                                            "__symbolic": "reference",
                                            "module": "../../core/change_detection/differs/iterable_differs",
                                            "name": "IterableDiffers"
                                        },
                                        {
                                            "__symbolic": "reference",
                                            "module": "../../core/change_detection/change_detector_ref",
                                            "name": "ChangeDetectorRef"
                                        }
                                    ]
                                }
                            ]
                        }
                    }
                }
            },
            '/tmp/angular2/src/core/linker/view_container_ref.d.ts': { "metadata": { "ViewContainerRef": { "__symbolic": "class" } } },
            '/tmp/angular2/src/core/linker/template_ref.d.ts': { "module": "./template_ref", "metadata": { "TemplateRef": { "__symbolic": "class" } } },
            '/tmp/angular2/src/core/change_detection/differs/iterable_differs.d.ts': { "metadata": { "IterableDiffers": { "__symbolic": "class" } } },
            '/tmp/angular2/src/core/change_detection/change_detector_ref.d.ts': { "metadata": { "ChangeDetectorRef": { "__symbolic": "class" } } },
            '/tmp/src/app/hero-detail.component.d.ts': {
                "__symbolic": "module",
                "metadata": {
                    "HeroDetailComponent": {
                        "__symbolic": "class",
                        "decorators": [
                            {
                                "__symbolic": "call",
                                "expression": {
                                    "__symbolic": "reference",
                                    "name": "Component",
                                    "module": "angular2/src/core/metadata"
                                },
                                "arguments": [
                                    {
                                        "selector": "my-hero-detail",
                                        "template": "\n  <div *ngIf=\"hero\">\n    <h2>{{hero.name}} details!</h2>\n    <div><label>id: </label>{{hero.id}}</div>\n    <div>\n      <label>name: </label>\n      <input [(ngModel)]=\"hero.name\" placeholder=\"name\"/>\n    </div>\n  </div>\n",
                                        "directives": [
                                            {
                                                "__symbolic": "reference",
                                                "name": "FORM_DIRECTIVES",
                                                "module": "angular2/src/common/forms/directives"
                                            }
                                        ]
                                    }
                                ]
                            }
                        ],
                        "members": {
                            "hero": [
                                {
                                    "__symbolic": "property",
                                    "decorators": [
                                        {
                                            "__symbolic": "call",
                                            "expression": {
                                                "__symbolic": "reference",
                                                "name": "Input",
                                                "module": "angular2/src/core/metadata"
                                            }
                                        }
                                    ]
                                }
                            ]
                        }
                    }
                }
            },
            '/src/extern.d.ts': { "__symbolic": "module", metadata: { s: "s" } }
        };
        return data[moduleId];
    };
    return MockReflectorHost;
}());
//# sourceMappingURL=static_reflector_spec.js.map